/* 
 * Copyright (C) 2008 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer <laird@lbreyer.com>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

#include "util.h"
#include "tessboxes.h"


/*@constant double M_LN2@*/

extern char *progname;
extern char *inputfile;
extern long inputline;
extern options_t u_options;
extern options_t m_options;
extern int cmd;

extern long system_pagesize;

int sa_signal = 0;
signal_cleanup_t cleanup = { NULL };

/***********************************************************
 * SIGNAL HANDLING                                         *
 ***********************************************************/
#if defined HAVE_SIGACTION
/* sigaction structure used by several functions */
struct sigaction act;
#endif

void my_sa_handler(int signum) {
#if defined HAVE_SIGACTION
  sa_signal = signum;
#endif
}

void sigsegv(int signum) {
  fprintf(stdout, "%s:error: segmentation fault at position %ld of %s\n", 
	  progname, inputline, inputfile);
  exit(0);
}

/* intercepts typical termination signals and tries to do the right thing */
void init_signal_handling() {
#if defined HAVE_SIGACTION

  memset(&act, 0, sizeof(act));

  /* set up global sigaction structure */

  act.sa_handler = my_sa_handler;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask,SIGHUP);
  sigaddset(&act.sa_mask,SIGINT);
  sigaddset(&act.sa_mask,SIGQUIT);
  sigaddset(&act.sa_mask,SIGTERM);
  sigaddset(&act.sa_mask,SIGPIPE);
  sigaddset(&act.sa_mask,SIGUSR1);
  act.sa_flags = 0;

  sigaction(SIGHUP, &act, NULL);
  sigaction(SIGINT, &act, NULL);
  sigaction(SIGQUIT, &act, NULL);
  sigaction(SIGTERM, &act, NULL);
  sigaction(SIGPIPE, &act, NULL);
  sigaction(SIGUSR1, &act, NULL);

  act.sa_handler = sigsegv;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask,SIGSEGV);
  act.sa_flags = 0;
  sigaction(SIGSEGV, &act, NULL);

#endif
}

void cleanup_signal_handling() {
  /* nothing - this is just to mess with your head ;-) */
}

void process_pending_signal(FILE *input) {
#if defined HAVE_SIGACTION
  
  if( sa_signal ) {

    sigprocmask(SIG_BLOCK, &act.sa_mask, NULL);
    switch(sa_signal) {
    case SIGINT:
      fprintf(stderr, 
	      "%s:signal: caught interrupt request, exiting\n", progname);
      exit(1);
      break;
    case SIGPIPE:
      /* should we terminate, or should we ignore? */
      fprintf(stderr,
	      "%s:error: broken pipe on output, exiting\n", progname);
      exit(1);
      break;
    case SIGHUP:
    case SIGQUIT:
    case SIGTERM:
      fprintf(stderr, 
	      "%s:signal: caught termination request, ignoring further input\n",
	      progname);
      if( input ) { fclose(input); }
      cmd |= FL_QUITNOW;
      break;
    case SIGUSR1:
      fprintf(stderr, 
	      "%s:signal: caught SIGUSR1 request, ignoring\n", progname);
      break;
    default:
      /* nothing */
      break;
    }
    sa_signal = 0;

    sigprocmask(SIG_UNBLOCK, &act.sa_mask, NULL);

  }

#endif
}


/***********************************************************
 * ERROR DISPLAY                                           *
 ***********************************************************/
void errormsg(int etype, const char *fmt, ...) {
  va_list vap;

  switch(etype) {
  case E_WARNING:
    fprintf(stderr, "%s:warning: ", progname);
    break;
  default:
  case E_ERROR:
  case E_FATAL:
    fprintf(stderr, "%s:error: ", progname);
    break;
  }

#if HAVE_VPRINTF
  va_start(vap, fmt);
  vfprintf(stderr, fmt, vap);
  va_end(vap);
#else
  fprintf(stderr, "%s", fmt);
#endif

  if( etype == E_FATAL ) { 
    exit(0);
  }
}

